import type {IBaseWorld} from '../common/world';
import type {Troonia} from './troonia';
import {SphereGeometry, Mesh, MeshPhongMaterial, Object3D, Scene} from 'three';
import {BaseWorld} from '../common/world';
import {Physics} from './physics';
import type {TSceneObject} from '../common/scenes';
import type {Socket} from 'socket.io-client';

export class World extends BaseWorld implements IBaseWorld {
  scene: Scene = new Scene();
  physics: Physics;
  player = {vec3: {x: 0, y: 0, z: 0}};

  constructor(game: Troonia) {
    super(game);
    this.physics = new Physics(this);
  }

  addPlayer(id?: string) {
    id = id ? id : (<Socket>this.game.socket).id;
    const player = new Object3D();
    player.name = id;
    const mesh = new Mesh(
        new SphereGeometry(.5), new MeshPhongMaterial({color: 0xFF00FF}));
    player.add(mesh);
    this.objects[id] = player;
    this.scene.add(player);
  }

  removePlayer(id?: string) {
    id = id ? id : (<Socket>this.game.socket).id;
    this.scene.remove(<Object3D>this.scene.getObjectByName(id));
    delete this.objects[id];
  }

  update(delta: number) {
    this.physics.update(delta);

    const player = <Object3D>this.objects[(<Socket>this.game.socket).id];
    (<Troonia>this.game).camera.lookAt(player.position);
    (<Troonia>this.game)
        .camera.position.set(player.position.x / 2, 50 - Math.abs(player.position.y), player.position.z / 2)
  }

  load(scene: TSceneObject[]) {
    for (const sceneObject of scene) {
      this.objects[sceneObject.id] = sceneObject.object;
      this.scene.add(sceneObject.object);
    }
  }
}