import {Body, Box, Shape, Vec3, World as PhysicsWorld} from 'cannon-es';
import type {World as GameWorld} from './world.js';
import type {IPhysicsObject} from '../common/physics.js';
import {BasePhysics} from '../common/physics.js';
import type {IVec3} from '../common/threed';

export class Physics extends BasePhysics {
  world: PhysicsWorld = new PhysicsWorld();

  constructor(baseWorld: GameWorld) {
    super(baseWorld);
    this.world.gravity.set(0, -9.82, 0);
  }

  addObject(
      id: string, object: IPhysicsObject, mass: number, size: IVec3,
      shape?: Shape) {
    const body = new Body({mass: mass});

    if (!shape) {
      body.addShape(new Box(new Vec3(size.x / 2, size.y / 2, size.z / 2)));
    } else {
      body.addShape(shape);
    }

    body.position.set(object.position.x, object.position.y, object.position.z);
    body.quaternion.set(
        object.quaternion.x, object.quaternion.y, object.quaternion.z,
        object.quaternion.w);

    this.bodies[id] = body;
    this.world.addBody(body);
  }

  removeObject(id: string) {
    this.world.removeBody(<Body>this.bodies[id]);
    delete this.bodies[id];
  }

  update(delta: number) {
    for (const player in (<GameWorld>this.baseWorld).players) {
      // (this.bodies[player] as Body)
      //     .angularVelocity.copy(
      //         (this.baseWorld as GameWorld).players[player]?.vec3 as Vec3);

      let px = this.baseWorld?.players[player]?.vec3?.x;
      let py = this.baseWorld?.players[player]?.vec3?.y;
      let pz = this.baseWorld?.players[player]?.vec3?.z;
      px = px ? px : 0;
      py = py ? py : 0;
      pz = pz ? pz : 0;
      if (px > 0) {
        (<Body>this.bodies[player]).angularVelocity.x += 1;
      }
      if (py > 0) {
        (<Body>this.bodies[player]).velocity.y += 0.5;
      }
      if (pz > 0) {
        (<Body>this.bodies[player]).angularVelocity.z += 1;
      }
      if (px < 0) {
        (<Body>this.bodies[player]).angularVelocity.x -= 1;
      }
      if (pz < 0) {
        (<Body>this.bodies[player]).angularVelocity.z -= 1;
      }
    }
    this.world.fixedStep(delta);
    this.syncState();
  }
}